@@ -201,35 +201,45 @@ List load_ply(std::string inputfile, std::string basedir) {
   single_shape["has_vertex_normals"] = LogicalVector(inds.size()/nv_face,inds.size() == norm_inds.size());
   shape_list[0]                = single_shape;
   
-  List out(26);
-  CharacterVector names(26) ;
+  const int num_items = 29;
   
-  set_item( out, 0 , _["ambient"]  =  NumericVector::create(0,0,0), names) ;
-  set_item( out, 1 , _["diffuse"]  = NumericVector::create(1,1,1), names) ;
-  set_item( out, 2 , _["specular"]  = NumericVector::create(0,0,0), names) ;
-  set_item( out, 3 , _["transmittance"]  = NumericVector::create(0,0,0), names) ;
-  set_item( out, 4 , _["emission"]  = NumericVector::create(0,0,0), names) ;
-  set_item( out, 5 , _["shininess"]  = 1.0, names) ;
-  set_item( out, 6 , _["ior"]  = 1.0, names) ;
-  set_item( out, 7 , _["dissolve"]  = 1.0, names) ;
-  set_item( out, 8 , _["illum"]  = 1, names) ;
-  set_item( out, 9 , _["ambient_texname"]    = "" ,  names) ;
-  set_item( out, 10, _["diffuse_texname"]    = "" ,  names) ;
-  set_item( out, 11, _["emissive_texname"]   = "",  names) ;
-  set_item( out, 12, _["specular_texname"]   = "",  names) ;
-  set_item( out, 13, _["normal_texname"]     = ""  ,  names) ;
-  set_item( out, 14, _["diffuse_intensity"]  = 1.0  , names) ;
-  set_item( out, 15, _["emission_intensity"] = 1.0, names) ;
-  set_item( out, 16, _["specular_intensity"] = 1.0, names) ;
-  set_item( out, 17, _["ambient_intensity"] = 1.0, names) ;
-  set_item( out, 18, _["culling"] = 1, names) ;
-  set_item( out, 19, _["type"] = "diffuse", names) ;
-  set_item( out, 20, _["translucent"] = true, names) ;
-  set_item( out, 21, _["toon_levels"] = 5, names) ;
-  set_item( out, 22, _["toon_outline_width"] = 0.05, names) ;
-  set_item( out, 23, _["toon_outline_color"] = "black", names) ;
-  set_item( out, 24, _["reflection_intensity"] = 1.0, names) ;
-  set_item( out, 25, _["reflection_sharpness"] = 1.0, names) ;
+  List out(num_items);
+  CharacterVector names(num_items) ;
+  int item = 0;
+  
+  set_item( out, item++, _["ambient"]  =  NumericVector::create(0,0,0), names) ;
+  set_item( out, item++, _["diffuse"]  = NumericVector::create(1,1,1), names) ;
+  set_item( out, item++, _["specular"]  = NumericVector::create(0,0,0), names) ;
+  set_item( out, item++, _["transmittance"]  = NumericVector::create(0,0,0), names) ;
+  set_item( out, item++, _["emission"]  = NumericVector::create(0,0,0), names) ;
+  set_item( out, item++, _["shininess"]  = 1.0, names) ;
+  set_item( out, item++, _["ior"]  = 1.0, names) ;
+  set_item( out, item++, _["dissolve"]  = 1.0, names) ;
+  set_item( out, item++, _["illum"]  = 1, names) ;
+  set_item( out, item++, _["ambient_texname"]    = "" ,  names) ;
+  set_item( out, item++, _["diffuse_texname"]    = "" ,  names) ;
+  set_item( out, item++, _["emissive_texname"]   = "",  names) ;
+  set_item( out, item++, _["specular_texname"]   = "",  names) ;
+  set_item( out, item++, _["normal_texname"]     = ""  ,  names) ;
+  set_item( out, item++, _["bump_texname"]    = "" ,  names) ;
+  set_item( out, item++, _["diffuse_intensity"]  = 1.0  , names) ;
+  set_item( out, item++, _["bump_intensity"]  = 1.0  , names) ;
+  set_item( out, item++, _["emission_intensity"] = 1.0, names) ;
+  set_item( out, item++, _["specular_intensity"] = 1.0, names) ;
+  set_item( out, item++, _["ambient_intensity"] = 1.0, names) ;
+  set_item( out, item++, _["culling"] = 1, names) ;
+  set_item( out, item++, _["type"] = "diffuse", names) ;
+  set_item( out, item++, _["translucent"] = true, names) ;
+  set_item( out, item++, _["toon_levels"] = 5, names) ;
+  set_item( out, item++, _["toon_outline_width"] = 0.05, names) ;
+  set_item( out, item++, _["toon_outline_color"] = NumericVector::create(0,0,0), names) ;
+  set_item( out, item++, _["reflection_intensity"] = 1.0, names) ;
+  set_item( out, item++, _["reflection_sharpness"] = 1.0, names) ;
+  set_item( out, item++, _["two_sided"] = false, names) ;
+  
+  if(item != num_items) {
+    throw std::runtime_error("Number of items is not equal to specified material length.");
+  }
   
   out.names() = names ;
   material_list[0] = out;
